@@ -1,7 +1,7 @@
 test_that("convert_to_omv works", {
     # check whether writing the data is working (file existence, size, contents [.omv-files are ZIP archives and must contain files that include meta, metadata.json, data.bin])
-    nmeInp <- paste0(tempfile(), ".rds")
-    nmeOut <- paste0(tempfile(), ".omv")
+    nmeInp <- tempfile(fileext = ".rds")
+    nmeOut <- tempfile(fileext = ".omv")
     saveRDS(datasets::ToothGrowth, nmeInp)
     convert_to_omv(fleInp = nmeInp, fleOut = nmeOut)
     expect_true(file.exists(nmeOut))
