@@ -78,7 +78,8 @@ test_that("read_omv works", {
     expect_error(read_omv(),   regexp = "^File name to the input data file needs to be given as parameter \\(fleInp = \\.\\.\\.\\)\\.")
     expect_error(read_omv(""), regexp = "^File name to the input data file needs to be given as parameter \\(fleInp = \\.\\.\\.\\)\\.")
     # fleInp is not a jamovi-file (.omv)
-    expect_error(read_omv("Trial.rds"), regexp = "^read_omv only reads jamovi files \\(\\.omv\\), use convert_to_omv first, if you want to read other files types\\.")
+    expect_error(read_omv("Trial.rds"),
+      regexp = "^read_omv only reads jamovi files \\(\\.omv\\ / \\.omt\\), use convert_to_omv first, if you want to read other files types\\.")
     # the manifest must have a file name as second parameter and
     # the file has to be a valid manifest file (which is not the
     # case for "index.html" [exists, but isn't a manifest])
@@ -100,6 +101,14 @@ test_that("read_omv works", {
     suppressMessages(expect_null(getHdl(fleOMV = nmeTmp, crrFle = "MANIFEST.MF")))
     unlink(nmeTmp)
 
+    # convert vector of 0 and 1 into logical
+    tmpCol <- sample(c(0, 1), 100, replace = TRUE)
+    cl4Chk <- valLbl(crrCol = tmpCol, mtaCol = list(name = "A", columnType = "Data", dataType = "Integer"),
+                     xtdDta = list(A = list(labels = list(list(0, "0", "0", FALSE), list(1, "1", "1", FALSE)))))
+    expect_true(is.logical(cl4Chk))
+    expect_null(attributes(cl4Chk))
+    expect_equal(c(mean(cl4Chk), sd(cl4Chk)), c(mean(tmpCol), sd(tmpCol)))
+
     # unimplemented columnType when assigning value labels
     expect_error(valLbl(mtaCol = list(name = "Trial", columnType = "Trial", dataType = "Trial"), xtdDta = list(Trial = NULL)),
       regexp = "Error when reading value label - likely the column type is not implemented \\(yet\\): Trial - Trial - Trial")
@@ -136,33 +145,39 @@ test_that("read_all works", {
     expect_error(read_all(),   regexp = "^File name to the input data file needs to be given as parameter \\(fleInp = \\.\\.\\.\\)\\.")
     expect_error(read_all(""), regexp = "^File name to the input data file needs to be given as parameter \\(fleInp = \\.\\.\\.\\)\\.")
 
-    # replace strings in attributes, etc.
-    # actual replacement
-    expect_equal(rplStr(strMod = "<c4><d6><dc><df><e4><f6><fc>", crrAtt = "Trial"), "ÄÖÜßäöü")
-    expect_error(rplStr(strMod = "<c3><28>", crrAtt = "Trial"),
-      regexp = "^The current data set still contains an invalid character \\(\".*\"\\) in attribute: \".*\"\\.")
-    # finding attributes that may contain such strings
-    tmpDF <- read_omv(nmeInp)
-    attr(tmpDF, "fltLst") <- "Filter <c4><d6><dc><df><e4><f6><fc>"
-    attr(tmpDF[["weights"]], "description") <- "<c4><d6><dc><df><e4><f6><fc>"
-    df4Chk <- rplAtt(dtaFrm = tmpDF)
-    expect_equal(attr(df4Chk, "fltLst"), "Filter ÄÖÜßäöü")
-    expect_equal(attr(df4Chk[["weights"]], "description"), "ÄÖÜßäöü")
-
     # more than one object when using Rdata
     D1 <- data.frame(A = runif(n = 100))
     D2 <- data.frame(B = runif(n = 100))
     nmeInp <- tempfile(fileext = ".rda")
     save(list = ls()[grepl("D[1-2]", ls())], file = nmeInp)
     # throw error when selSet is not sepcified
-    suppressMessages(expect_error(read_all(nmeInp),
-      regexp = "^Input data are either not a data frame or have incorrect \\(only one or more than two\\) dimensions\\."))
+    expect_message(expect_null(read_all(nmeInp)), regexp = "^File \".*\" couldn't be read\\.\nThe error message was: The Rdata-file must include only one object\\.")
     # check whether reading works correct with using selSet
     df4Chk <- read_all(nmeInp, selSet = "D1")
     expect_s3_class(df4Chk, class = "data.frame")
     expect_equal(dim(df4Chk), c(100, 1))
     unlink(nmeInp)
 
+    # test cases for clnTbl in RData and RDS
+    fleInp <- tempfile()
+    if (requireNamespace("haven", quietly = TRUE)) {
+        haven::write_sav(jmvReadWrite::ToothGrowth, paste0(fleInp, ".sav"))
+        tmpDta <- haven::read_sav(paste0(fleInp, ".sav"))
+        unlink(paste0(fleInp, ".sav"))
+        save(tmpDta, file = paste0(fleInp, ".RData"))
+        df4Chk <- read_all(paste0(fleInp, ".RData"))
+        expect_equal(attributes(df4Chk), list(row.names = seq(60), names = c("ID", "supp", "supp2", "dose", "dose2", "len", "logLen"), class = "data.frame"))
+        expect_equal(lapply(df4Chk, attributes), list(ID = NULL, supp = list(levels = c("OJ", "VC"), class = "factor"), supp2 = list(levels = c("1", "2"), class = "factor"),
+                                                      dose = NULL, dose2 = list(levels = c("0.5", "1.0", "2.0"), class = "factor"), len = NULL, logLen = NULL))
+        unlink(paste0(fleInp, ".RData"))
+        saveRDS(tmpDta, file = paste0(fleInp, ".rds"))
+        df4Chk <- read_all(paste0(fleInp, ".rds"))
+        expect_equal(attributes(df4Chk), list(row.names = seq(60), names = c("ID", "supp", "supp2", "dose", "dose2", "len", "logLen"), class = "data.frame"))
+        expect_equal(lapply(df4Chk, attributes), list(ID = NULL, supp = list(levels = c("OJ", "VC"), class = "factor"), supp2 = list(levels = c("1", "2"), class = "factor"),
+                                                      dose = NULL, dose2 = list(levels = c("0.5", "1.0", "2.0"), class = "factor"), len = NULL, logLen = NULL))
+        unlink(paste0(fleInp, ".rds"))
+    }
+
     # test cases for CSV / TSV
     nmeInp <- tempfile(fileext = ".tsv")
     write.table(D1, file = nmeInp, sep = "\t")
@@ -190,53 +205,123 @@ test_that("read_all works", {
 
     nmeInp <- tempfile(fileext = ".csv")
     writeBin("X1,X2\n1.2,2.2\n7.1,3.2", nmeInp)
-    suppressMessages(expect_error(read_all(nmeInp),
-      regexp = "^Input data are either not a data frame or have incorrect \\(only one or more than two\\) dimensions\\."))
+    expect_message(expect_null(read_all(nmeInp)),
+      regexp = "^Warnings were issued when reading the file \".*\"\\.\nThe warning was: line 3 appears to contain embedded nulls")
     unlink(nmeInp)
 
     fleInp <- tempfile(fileext = ".sav")
     writeBin("$FL2@(#) IBM SPSS STATISTICS 64-bit Linux 25.0.0.0              \002", con = fleInp)
-    suppressMessages(expect_error(read_all(fleInp, usePkg = "haven"),
-      regexp = "^Input data are either not a data frame or have incorrect \\(only one or more than two\\) dimensions\\."))
-    suppressMessages(expect_error(read_all(fleInp, usePkg = "foreign"),
-      regexp = "^Input data are either not a data frame or have incorrect \\(only one or more than two\\) dimensions\\."))
+    expect_message(expect_null(read_all(fleInp, usePkg = "haven")),
+      regexp = "^File \".*\" couldn't be read\\.\nThe error message was: Failed to parse .*: Unable to read from file\\.")
+    expect_message(expect_null(read_all(fleInp, usePkg = "foreign")), regexp = "^File \".*\" couldn't be read\\.")
     unlink(fleInp)
+    if (requireNamespace("haven", quietly = TRUE)) {
+        haven::write_sav(jmvReadWrite::ToothGrowth, fleInp)
+        df4Chk <- read_all(fleInp, usePkg = "haven")
+        expect_equal(attributes(df4Chk), list(row.names = seq(60), names = c("ID", "supp", "supp2", "dose", "dose2", "len", "logLen"), class = "data.frame"))
+        expect_equal(lapply(df4Chk, attributes), list(ID = NULL, supp = list(levels = c("OJ", "VC"), class = "factor"), supp2 = list(levels = c("1", "2"), class = "factor"),
+                                                      dose = NULL, dose2 = list(levels = c("0.5", "1.0", "2.0"), class = "factor"), len = NULL, logLen = NULL))
+        unlink(fleInp)
+    }
 
     fleInp <- tempfile(fileext = ".dta")
     writeBin("<stata_dta><header><release>117</release><byteorder>LSF</byteorder><K>\x8f", con = fleInp)
-    suppressMessages(expect_error(read_all(fleInp, usePkg = "haven"),
-      regexp = "^Input data are either not a data frame or have incorrect \\(only one or more than two\\) dimensions\\."))
-    writeBin("", con = fleInp)
-    suppressMessages(expect_error(read_all(fleInp, usePkg = "foreign"),
-      regexp = "^Input data are either not a data frame or have incorrect \\(only one or more than two\\) dimensions\\."))
+    suppressWarnings(expect_message(expect_null(read_all(fleInp, usePkg = "haven")),
+      regexp = "File \".*\" couldn't be read\\.\nThe error message was: Failed to parse .*: Unable to read from file\\."))
+    suppressWarnings(expect_message(expect_null(read_all(fleInp, usePkg = "foreign")),
+      regexp = "File \".*\" couldn't be read\\.\nThe error message was: not a Stata version 5-12 .dta file"))
     unlink(fleInp)
+    if (requireNamespace("haven", quietly = TRUE)) {
+        haven::write_dta(jmvReadWrite::ToothGrowth, fleInp)
+        df4Chk <- read_all(fleInp, usePkg = "haven")
+        expect_equal(attributes(df4Chk), list(row.names = seq(60), names = c("ID", "supp", "supp2", "dose", "dose2", "len", "logLen"), class = "data.frame"))
+        expect_equal(lapply(df4Chk, attributes), list(ID = NULL, supp = list(levels = c("OJ", "VC"), class = "factor"), supp2 = list(levels = c("1", "2"), class = "factor"),
+                                                      dose = NULL, dose2 = list(levels = c("0.5", "1.0", "2.0"), class = "factor"), len = NULL, logLen = NULL))
+        unlink(fleInp)
+    }
 
     fleInp <- tempfile(fileext = ".sas7bdat")
     writeBin("", con = fleInp)
-    suppressMessages(expect_error(capture_output(read_all(fleInp, usePkg = "haven")),
-      regexp = "^Input data are either not a data frame or have incorrect \\(only one or more than two\\) dimensions\\."))
-    suppressMessages(expect_error(read_all(fleInp, usePkg = "foreign"),
-      regexp = "^Input data are either not a data frame or have incorrect \\(only one or more than two\\) dimensions\\."))
+    invisible(capture.output(expect_message(expect_null(read_all(fleInp, usePkg = "haven")), "File \".*\" couldn't be read")))
+    expect_error(read_all(fleInp, usePkg = "foreign"), "In order to read the SAS-file \".*\" the R-packages \"haven\" needs to be installed\\.")
     unlink(fleInp)
+    if (requireNamespace("haven", quietly = TRUE)) {
+        suppressWarnings(haven::write_sas(jmvReadWrite::ToothGrowth, fleInp))
+        df4Chk <- read_all(fleInp, usePkg = "haven")
+        expect_equal(attributes(df4Chk), list(class = "data.frame", row.names = seq(60), names = c("ID", "supp", "supp2", "dose", "dose2", "len", "logLen")))
+        expect_equal(lapply(df4Chk, attributes), list(ID = NULL, supp = NULL, supp2 = NULL, dose = NULL, dose2 = NULL, len = NULL, logLen = NULL))
+        unlink(fleInp)
+    }
 
     fleInp <- tempfile(fileext = ".xpt")
     writeBin("HEADER RECORD*******LIBRARY HEADER RECORD!!!!!!!", con = fleInp)
-    suppressMessages(expect_error(read_all(fleInp, usePkg = "haven"),
-      regexp = "^Input data are either not a data frame or have incorrect \\(only one or more than two\\) dimensions\\."))
-    suppressMessages(expect_error(read_all(fleInp, usePkg = "foreign"),
-      regexp = "^Input data are either not a data frame or have incorrect \\(only one or more than two\\) dimensions\\."))
+    expect_message(expect_null(read_all(fleInp, usePkg = "haven")),
+      regexp = "^File \".*\" couldn't be read\\.\nThe error message was: Failed to parse .*: Unable to read from file\\.")
+    expect_message(expect_null(read_all(fleInp, usePkg = "foreign")),
+      regexp = "^File \".*\" couldn't be read\\.\nThe error message was: file not in SAS transfer format")
     unlink(fleInp)
+    if (requireNamespace("haven", quietly = TRUE)) {
+        haven::write_xpt(jmvReadWrite::ToothGrowth, fleInp)
+        df4Chk <- read_all(fleInp, usePkg = "haven")
+        expect_equal(attributes(df4Chk), list(class = "data.frame", row.names = seq(60), names = c("ID", "supp", "supp2", "dose", "dose2", "len", "logLen")))
+        expect_equal(lapply(df4Chk, attributes), list(ID = NULL, supp = NULL, supp2 = NULL, dose = NULL, dose2 = NULL, len = NULL, logLen = NULL))
+        unlink(fleInp)
+    }
 
     dtaTmp <- jmvReadWrite::ToothGrowth
-    expect_null(attributes(hvnAdj(dtaTmp, c("jmv-id", "jmv-desc"))[[1]]))
-    expect_null(attributes(hvnAdj(dtaTmp, c("jmv-id", "jmv-desc"))[[7]]))
+    expect_null(attributes(clnTbb(dtaTmp, c("jmv-id", "jmv-desc"))[[1]]))
+    expect_null(attributes(clnTbb(dtaTmp, c("jmv-id", "jmv-desc"))[[7]]))
     attr(dtaTmp[[6]], "label") <- "Trial for label conversion"
-    expect_equal(attributes(hvnAdj(dtaTmp, jmvLbl = TRUE)[[6]]), list(`jmv-desc` = "Trial for label conversion"))
+    expect_equal(attributes(clnTbb(dtaTmp, jmvLbl = TRUE)[[6]]), list(`jmv-desc` = "Trial for label conversion"))
+
+    set.seed(2)
+    dtaTmp <- structure(list(value = structure(sample(c(1, 2, 4), 200, replace = TRUE), format.sas = "LEVELS", class = c("haven_labelled", "vctrs_vctr", "double"),
+                             labels = c(`Level 1` = 1, `Level 2` = 2, `Level 3` = 4))), class = c("tbl_df", "tbl", "data.frame"), row.names = c(NA, -200L))
+    expect_s3_class(clnTbb(dtaTmp), "data.frame")
+    expect_s3_class(clnTbb(dtaTmp)[, "value"], "factor")
+    expect_equal(attributes(clnTbb(dtaTmp)), list(names = "value", row.names = seq(200), class = "data.frame"))
+    expect_equal(attributes(clnTbb(dtaTmp)[, "value"]), list(levels = sprintf("Level %d", seq(3)), class = "factor", format.sas = "LEVELS"))
+    expect_equal(attributes(clnTbb(dtaTmp, rmvAtt = "format.sas")[, "value"]), list(levels = sprintf("Level %d", seq(3)), class = "factor"))
+    expect_equal(as.integer(table(clnTbb(dtaTmp)[, "value"])), c(71, 69, 60))
+    expect_equal(names(table(clnTbb(dtaTmp)[, "value"])), sprintf("Level %d", seq(3)))
+
+    dtaTmp <- structure(list(value = structure(sample(c(1, 2, 4), 200, replace = TRUE), format.sas = "LEVELS", class = c("haven_labelled", "vctrs_vctr", "double"),
+                             labels = setNames(c(1, 2, 4), rep("", 3)))), class = c("tbl_df", "tbl", "data.frame"), row.names = c(NA, -200L))
+    expect_s3_class(clnTbb(dtaTmp), "data.frame")
+    expect_type(clnTbb(dtaTmp)[, "value"], "integer")
+    expect_equal(attributes(clnTbb(dtaTmp)), list(names = "value", row.names = seq(200), class = "data.frame"))
+    expect_equal(attributes(clnTbb(dtaTmp)[, "value"]), list(format.sas = "LEVELS"))
+    expect_null(attributes(clnTbb(dtaTmp, rmvAtt = "format.sas")[, "value"]))
+    expect_equal(as.integer(clnTbb(dtaTmp)[, "value"])[1:10], c(4, 4, 2, 2, 4, 2, 4, 1, 2, 4))
+    expect_equal(as.integer(table(clnTbb(dtaTmp)[, "value"])), c(66, 67, 67))
+    expect_equal(names(table(clnTbb(dtaTmp)[, "value"])), sprintf("%d", c(1, 2, 4)))
+
+    dtaTmp <- structure(list(value = structure(sample(c(1, 2, 4), 200, replace = TRUE), format.sas = "LEVELS", class = c("haven_labelled", "vctrs_vctr", "double"),
+                             labels = setNames(c(1, 2, 4), rep("value label", 3)))), class = c("tbl_df", "tbl", "data.frame"), row.names = c(NA, -200L))
+    expect_s3_class(clnTbb(dtaTmp), "data.frame")
+    expect_type(clnTbb(dtaTmp)[, "value"], "integer")
+    expect_equal(attributes(clnTbb(dtaTmp)), list(names = "value", row.names = seq(200), class = "data.frame"))
+    expect_equal(attributes(clnTbb(dtaTmp)[, "value"]), list(format.sas = "LEVELS"))
+    expect_null(attributes(clnTbb(dtaTmp, rmvAtt = "format.sas")[, "value"]))
+    expect_equal(as.integer(clnTbb(dtaTmp)[, "value"])[1:10], c(1, 1, 2, 1, 1, 4, 1, 2, 1, 1))
+    expect_equal(as.integer(table(clnTbb(dtaTmp)[, "value"])), c(70, 72, 58))
+    expect_equal(names(table(clnTbb(dtaTmp)[, "value"])), sprintf("%d", c(1, 2, 4)))
+
+    dtaTmp <- structure(list(value = structure(sample(seq(4), 200, replace = TRUE), format.sas = "LEVELS", class = c("haven_labelled", "vctrs_vctr", "double"),
+                             labels = setNames(seq(4), c("Smallest", "", "", "Largest")))), class = c("tbl_df", "tbl", "data.frame"), row.names = c(NA, -200L))
+    expect_s3_class(clnTbb(dtaTmp), "data.frame")
+    expect_s3_class(clnTbb(dtaTmp)[, "value"], "factor")
+    expect_equal(attributes(clnTbb(dtaTmp)), list(names = "value", row.names = seq(200), class = "data.frame"))
+    expect_equal(attributes(clnTbb(dtaTmp)[, "value"]), list(levels = c("Smallest", "2", "3", "Largest"), class = "factor", format.sas = "LEVELS"))
+    expect_equal(attributes(clnTbb(dtaTmp, rmvAtt = "format.sas")[, "value"]), list(levels = c("Smallest", "2", "3", "Largest"), class = "factor"))
+    expect_equal(as.integer(clnTbb(dtaTmp)[, "value"])[1:10], c(4, 2, 1, 3, 2, 1, 3, 3, 2, 2))
+    expect_equal(as.integer(table(clnTbb(dtaTmp)[, "value"])), c(52, 47, 46, 55))
+    expect_equal(names(table(clnTbb(dtaTmp)[, "value"])), c("Smallest", "2", "3", "Largest"))
 
     dtaTmp <- jmvReadWrite::AlbumSales[-1]
     attr(dtaTmp, "variable.labels") <- c(Adverts = "Advertsing budget (thousands)",
                                          Airplay = "No. of plays on radio",
                                          Image = "Band image rating (0-10)",
                                          Sales = "Album sales (thousands)")
-    expect_identical(vapply(fgnLbl(dtaTmp), attr, character(1), "jmv-desc"), attr(dtaTmp, "variable.labels"))
+    expect_identical(vapply(clnFgn(dtaTmp), attr, character(1), "jmv-desc"), attr(dtaTmp, "variable.labels"))
 })
